;============================================================================
; DBGMSGS.ASM
;
; Tui messaging and thread debug/display functions.
;============================================================================




.data




; Output message.

szOut		db	256 dup (0)


; The Activ/DeActiv strings expect up to 20 char of a window title
; or class name to be appended. As these strings are 22 char, 2 char are
; added - 1 for terminator and one to pad to dword boundary (44 char).

nDbg_WAct	equ	22
nDbg_Title	equ	20

szDbg_TMActiv	db	"TWndActiv, DeActWnd = "
		db	nDbg_Title+2 dup (0)
szDbg_TMDeAct	db	"TWndDeAct, ActivWnd = "
		db	nDbg_Title+2 dup (0)

szDbg_SBActiv	db	"SBarActiv, DeActWnd = "
		db	nDbg_Title+2 dup (0)
szDbg_SBDeAct	db	"SBarDeAct, ActivWnd = "
		db	nDbg_Title+2 dup (0)

szDbg_FDActiv	db	"FindActiv, DeActWnd = "
		db	nDbg_Title+2 dup (0)
szDbg_FDDeAct	db	"FindDeAct, ActivWnd = "
		db	nDbg_Title+2 dup (0)

szDbg_GPActiv	db	"GrupActiv, DeActWnd = "
		db	nDbg_Title+2 dup (0)
szDbg_GPDeAct	db	"GrupDeAct, ActivWnd = "
		db	nDbg_Title+2 dup (0)

szDbg_EXActiv	db	"ExptActiv, DeActWnd = "
		db	nDbg_Title+2 dup (0)
szDbg_EXDeAct	db	"ExptDeAct, ActivWnd = "
		db	nDbg_Title+2 dup (0)

szDbg_Desk	db	"DeskTop(?)",0		; if no window handle


szDbg_GNxNdx	db	"DB_GetNfo, BadNdx, ndx = ",0


szDbg_UDBEntry	db	"UDB Entry",0
szDbg_UDBExit	db	"UDB Exit",0
szDbg_UDBWaitC	db	"UDB Waiting for Change notification...",0
szDbg_UDBStCre	db	"UDB DB create Start",0
szDbg_UDBEnCre	db	"UDB DB create End",0
szDbg_UDBWaitU	db	"UDB Waiting, seconds = ",0
szDbg_UDBUpd	db	" -> DB Updated",0



.code




;----------------------------------------------------------------------------
Dbg_UDBStatus	proc
;------------
;
; Load the UDBStatus flags to the output message.
;----------------------------------------------------------------------------


	mov	eax, G_UDBStatus
	shl	eax, 24
	lea	ebx, szOut
	xor	ecx, ecx
	mov	edx, '01'

	.repeat

		shl	eax, 1

		.if	(CARRY?)
			mov	[ebx + ecx], dl
		.else
			mov	[ebx + ecx], dh
		.endif

		inc	ecx

	.until	(ecx == 8)		; 8 bits

	mov	eax, ' '		; space + terminator
	mov	[ebx + ecx], eax


	ret


Dbg_UDBStatus	endp
;----------------------------------------------------------------------------




;----------------------------------------------------------------------------
Dbg_WrWMAct	proc	pszMsg:dword,hWndP:dword
;----------
;
; Write an Activate/Deactivate message to log file, appending prior window
; Title.
;
; Local function, use internally only.
;
;
; Entry		pszMsg		pointer to start of message
;		hWndP		hwndPrevious
;
;
; Api note:
;
; WM_ACTIVATE lParam is value of hwndPrevious which identifies the window
; being activated or deactivated, depending on the value of the fActive
; parameter. If the value of fActive is WA_INACTIVE, hwndPrevious is the
; handle of the window being activated. If the value of fActive is WA_ACTIVE
; or WA_CLICKACTIVE, hwndPrevious is the handle of the window being
; deactivated. This handle can be NULL.
;----------------------------------------------------------------------------


	mov	ebx, pszMsg
	add	ebx, nDbg_WAct
	mov	eax, hWndP

	.if	(eax)

		; Have a handle.

		invoke	GetWindowText, eax, ebx, nDbg_Title
		add	ebx, eax
		mov	byte ptr [ebx], 0		; terminator

	.else

		; No handle, probably DeskTop.

		mov	byte ptr [ebx], 0		; need for cat
		invoke	szCatStr, ebx, addr szDbg_Desk

	.endif


	invoke	szCatStr, addr szOut, pszMsg
	invoke	Dbg_LogWrite, addr szOut


	ret


Dbg_WrWMAct	endp
;----------------------------------------------------------------------------




;----------------------------------------------------------------------------
Dbg_TM_WMAct	proc	Activate:dword,lParam:dword
;-----------
;
; Time window WM_ACTIVATE
;----------------------------------------------------------------------------


	pushad

	invoke	Dbg_UDBStatus

	.if	(Activate)
		mov	ebx, offset szDbg_TMActiv
	.else
		mov	ebx, offset szDbg_TMDeAct
	.endif

	invoke	Dbg_WrWMAct, ebx, lParam

	popad


	ret


Dbg_TM_WMAct	endp
;----------------------------------------------------------------------------




;----------------------------------------------------------------------------
Dbg_SB_WMAct	proc	Activate:dword,lParam:dword
;-----------
;
; SideBar WM_ACTIVATE
;----------------------------------------------------------------------------


	pushad

	invoke	Dbg_UDBStatus

	.if	(Activate)
		mov	ebx, offset szDbg_SBActiv
	.else
		mov	ebx, offset szDbg_SBDeAct
	.endif

	invoke	Dbg_WrWMAct, ebx, lParam

	popad


	ret


Dbg_SB_WMAct	endp
;----------------------------------------------------------------------------




;----------------------------------------------------------------------------
Dbg_FD_WMAct	proc	Activate:dword,lParam:dword
;-----------
;
; Find Dialog WM_ACTIVATE
;----------------------------------------------------------------------------


	pushad

	invoke	Dbg_UDBStatus

	.if	(Activate)
		mov	ebx, offset szDbg_FDActiv
	.else
		mov	ebx, offset szDbg_FDDeAct
	.endif

	invoke	Dbg_WrWMAct, ebx, lParam

	popad


	ret


Dbg_FD_WMAct	endp
;----------------------------------------------------------------------------




;----------------------------------------------------------------------------
Dbg_GP_WMAct	proc	Activate:dword,lParam:dword
;-----------
;
; Group Dialog WM_ACTIVATE
;----------------------------------------------------------------------------


	pushad

	invoke	Dbg_UDBStatus

	.if	(Activate)
		mov	ebx, offset szDbg_GPActiv
	.else
		mov	ebx, offset szDbg_GPDeAct
	.endif

	invoke	Dbg_WrWMAct, ebx, lParam

	popad


	ret


Dbg_GP_WMAct	endp
;----------------------------------------------------------------------------




;----------------------------------------------------------------------------
Dbg_EX_WMAct	proc	Activate:dword,lParam:dword
;-----------
;
; Export Dialog WM_ACTIVATE
;----------------------------------------------------------------------------


	pushad

	invoke	Dbg_UDBStatus

	.if	(Activate)
		mov	ebx, offset szDbg_EXActiv
	.else
		mov	ebx, offset szDbg_EXDeAct
	.endif

	invoke	Dbg_WrWMAct, ebx, lParam

	popad


	ret


Dbg_EX_WMAct	endp
;----------------------------------------------------------------------------




;----------------------------------------------------------------------------
Dbg_DB_GNxNdx	proc	Ndx:dword
;------------
;
; DB_GetNfo, bad index at entry.
;----------------------------------------------------------------------------


	local	szConv[16]		:byte


	pushad

	invoke	szCopy, addr szDbg_GNxNdx, addr szOut
	invoke	dwtoa, Ndx, addr szConv
	invoke	szCatStr, addr szOut, addr szConv
	invoke	Dbg_LogWrite, addr szOut

	popad


	ret


Dbg_DB_GNxNdx	endp
;----------------------------------------------------------------------------




;----------------------------------------------------------------------------
Dbg_UDBEntry	proc
;-----------
;
; Update database thread entered.
;----------------------------------------------------------------------------


	pushad

	invoke	Dbg_UDBStatus
	invoke	szCatStr, addr szOut, addr szDbg_UDBEntry

	invoke	Dbg_LogWrite, addr szOut

	popad


	ret


Dbg_UDBEntry	endp
;----------------------------------------------------------------------------




;----------------------------------------------------------------------------
Dbg_UDBExit	proc
;----------
;
; Update database thread exiting.
;----------------------------------------------------------------------------


	pushad

	invoke	Dbg_UDBStatus
	invoke	szCatStr, addr szOut, addr szDbg_UDBExit

	invoke	Dbg_LogWrite, addr szOut

	popad


	ret


Dbg_UDBExit	endp
;----------------------------------------------------------------------------




;----------------------------------------------------------------------------
Dbg_UDBWaitChg	proc
;-------------
;
; Update database wait for change notification.
;----------------------------------------------------------------------------


	pushad

	invoke	Dbg_UDBStatus
	invoke	szCatStr, addr szOut, addr szDbg_UDBWaitC

	invoke	Dbg_LogWrite, addr szOut

	popad


	ret


Dbg_UDBWaitChg	endp
;----------------------------------------------------------------------------




;----------------------------------------------------------------------------
Dbg_UDBStCre	proc
;-----------
;
; Update database start db creation.
;----------------------------------------------------------------------------


	pushad

	invoke	Dbg_UDBStatus
	invoke	szCatStr, addr szOut, addr szDbg_UDBStCre

	invoke	Dbg_LogWrite, addr szOut

	popad


	ret


Dbg_UDBStCre	endp
;----------------------------------------------------------------------------




;----------------------------------------------------------------------------
Dbg_UDBEnCre	proc
;-----------
;
; Update database db creation end.
;----------------------------------------------------------------------------


	pushad

	invoke	Dbg_UDBStatus
	invoke	szCatStr, addr szOut, addr szDbg_UDBEnCre

	invoke	Dbg_LogWrite, addr szOut

	popad


	ret


Dbg_UDBEnCre	endp
;----------------------------------------------------------------------------




;----------------------------------------------------------------------------
Dbg_UDBWaitUpd	proc	Cnt:dword,Upd:dword
;-------------
;
; Update database update wait status.
;
;
; Entry		Cnt		second counter
;
;		Upd		true if update has occurred
;----------------------------------------------------------------------------


	local	szConv[16]		:byte


	pushad

	invoke	Dbg_UDBStatus
	invoke	szCatStr, addr szOut, addr szDbg_UDBWaitU

	invoke	dwtoa, Cnt, addr szConv
	invoke	szCatStr, addr szOut, addr szConv

	.if	(Upd)
		invoke	szCatStr, addr szOut, addr szDbg_UDBUpd
	.endif

	invoke	Dbg_LogWrite, addr szOut

	popad


	ret


Dbg_UDBWaitUpd	endp
;----------------------------------------------------------------------------




;============================================================================
