;============================================================================
; SBARPUB.ASM
;
; Sidebar module public functions.
;============================================================================




;----------------------------------------------------------------------------
SBar_Init	proc
;--------
;
; Sidebar initialisation.
;
;
; Entry,Exit	n/a
;
;
; The Sidebar is minimised and the Time window set foremost at WM_CREATE.
;----------------------------------------------------------------------------


	local	wc		:WNDCLASSEX


; Get the Time window handle.
;
; We must do this now as creation of the SideBar window calls WM_CREATE and
; we need the handle then.


	invoke	Time_Handle
	mov	L_hTimeWnd, eax


; Sidebar window (see Time window creation note).
;
; Note there is no need for a background or border as the ToolBar provides
; it - and in fact draws over any that are specified.


	mov	wc.cbSize,		sizeof WNDCLASSEX
	mov	wc.style,		CS_DBLCLKS
	mov	wc.lpfnWndProc, 	offset SBarWndHdlr
	mov	wc.cbClsExtra,		NULL
	mov	wc.cbWndExtra,		NULL
	mov	eax, 			G_hTui
	mov	wc.hInstance,		eax
	mov	wc.hIcon, 		NULL
	mov	wc.hIconSm,		NULL
	invoke	LoadCursorA, 		NULL, IDC_ARROW
	mov	wc.hCursor,		eax
	mov	wc.hbrBackground,	NULL
	mov	wc.lpszMenuName,	NULL
	mov	wc.lpszClassName,	offset szSBarCN

	invoke	RegisterClassExA, addr wc

	.if	(eax == NULL)			; Register Class failed

		invoke	GetLastError
		mov	G_APIErrCode, eax

		invoke	RaiseException, ERR_WndReg,
				0, nErrors, addr ErrorMsgs

	.endif

	mov	eax, G_ScrnWid
	sub	eax, SbarWidth
	mov	SBarXLeft, eax			; x coordinate

	invoke	CreateWindowExA, WS_EX_TOPMOST or WS_EX_TOOLWINDOW,
				addr szSBarCN,
				addr szSBarCN,
				WS_POPUP,
				eax, 0, SbarWidth, G_ScrnHgt,
				NULL, NULL,
				G_hTui, NULL

	mov	hSBarWnd, eax

	.if	(eax == NULL)			; Create failed

		invoke	GetLastError
		mov	G_APIErrCode, eax

		invoke	RaiseException, ERR_WndCreate,
				0, nErrors, addr ErrorMsgs

	.endif


; Vertical ToolBar.


TBStyle	equ	WS_VISIBLE or WS_CHILD or WS_BORDER or TBSTYLE_TOOLTIPS or \
		TBSTYLE_WRAPABLE or CCS_VERT	; (or TBSTYLE_FLAT)

	invoke	CreateToolbarEx, eax, TBStyle, ID_SB_TB, kTBButtons,
			G_hTui,	ID_SB_TBBMP, addr TBButtons, kTBButtons,
			0, 0, TBBmpXY, TBBmpXY, sizeof TBBUTTON

	mov	hTBWnd, eax

	.if	(eax == NULL)			; Create failed

		invoke	GetLastError
		mov	G_APIErrCode, eax

		invoke	RaiseException, ERR_TBarCreate,
				0, nErrors, addr ErrorMsgs

	.endif


; Exit.


	ret


SBar_Init	endp
;----------------------------------------------------------------------------




;----------------------------------------------------------------------------
SBar_DeInit	proc
;----------
;
; De-Initialise SideBar module objects.
;
;
; Entry,Exit	n/a
;----------------------------------------------------------------------------


	ret


SBar_DeInit	endp
;----------------------------------------------------------------------------




;----------------------------------------------------------------------------
SBar_Destroy	proc
;-----------
;
; Destroy SideBar window.
;
; NB this cannot go in SBar_DeInit as it uses the Tui message loop.
;
; Check handle is not zero jic we are coming here from the Fatal Execption
; handler and we are cleaning up after the user has dismissed the MessageBox.
;
;
; Entry,Exit	n/a
;
;
; DestroyWindow does not exit until all its messages are sent and processed.
;----------------------------------------------------------------------------


	.if	(hSBarWnd != 0)

		invoke	DestroyWindow, hSBarWnd

	.endif


	ret


SBar_Destroy	endp
;----------------------------------------------------------------------------




;----------------------------------------------------------------------------
SBar_DeskUp	proc
;----------
;
; Minimise Sidebar/Time window and show Desktop.
;
;
; Entry,Exit	n/a
;----------------------------------------------------------------------------


	; Use SendMessage rather than PostMessage so it is done when we
	; return from here.

	invoke	SendMessageA, hSBarWnd, WM_COMMAND, ID_SB_BTDESK, NULL


	ret


SBar_DeskUp	endp
;----------------------------------------------------------------------------




;============================================================================
