;============================================================================
; TIMEPUB.ASM
;
; Time module public functions.
;============================================================================




;----------------------------------------------------------------------------
Time_Init	proc	uses ebx
;--------
;
; Initialise Time module.
;
;
; Entry		n/a
;
;
; Exit		External vars initialised
;
;
; The Time window is not shown until the SideBar message handler receives
; WM_CREATE, which posts a Time button pressed message which then calls
; ShowWindow (Time window).
;----------------------------------------------------------------------------


	local	IniBuf[16]	:byte			; Ini key buffer
	local	wc		:WNDCLASSEX


; Load Color set.
;
; Need these now for Time window create below.


	; Query ini file for color section.
	; If no ini file or no color section then load defaults.

	invoke	File_TuiQryIni

	.if	(!CARRY?)			; ini found

		; Read color values.

		invoke	File_RdIniKey, addr szTMIS_COLR, addr szTMIK_TWBG,
				addr IniBuf, sizeof IniBuf
		.if	(eax)
			invoke	htodw, addr IniBuf
		.else
			mov	eax, DefCTime
		.endif
		mov	CTimeWnd, eax

		invoke	File_RdIniKey, addr szTMIS_COLR, addr szTMIK_ITBG,
				addr IniBuf, sizeof IniBuf
		.if	(eax)
			invoke	htodw, addr IniBuf
		.else
			mov	eax, DefCItem
		.endif
		mov	CTimeItem, eax

		invoke	File_RdIniKey, addr szTMIS_COLR, addr szTMIK_TSTX,
				addr IniBuf, sizeof IniBuf
		.if	(eax)
			invoke	htodw, addr IniBuf
		.else
			mov	eax, DefCTS
		.endif
		mov	CTimeStamp, eax

		invoke	File_RdIniKey, addr szTMIS_COLR, addr szTMIK_TBBG,
				addr IniBuf, sizeof IniBuf
		.if	(eax)
			invoke	htodw, addr IniBuf
		.else
			mov	eax, DefTimeBox
		.endif
		mov	CTimeBox, eax

	.else

		mov	eax, DefCTime
		mov	CTimeWnd, eax
		mov	eax, DefCItem
		mov	CTimeItem, eax
		mov	eax, DefCTS
		mov	CTimeStamp, eax
		mov	eax, DefTimeBox
		mov	CTimeBox, eax

	.endif

	; Mark as allocated, not because they need de-allocation but
	; in case of an exit before here - we don't want blank colors
	; stored to the ini file in Time_DeInit.

	or	TM_OAF, TM_OAF_Colors


; Load Icons.
;
; Have to use LoadImage so Windows doesn't scale out to 32x32!.


	invoke	LoadImage, G_hTui, ID_TM_UPICON, IMAGE_ICON, 0, 0,
			LR_DEFAULTCOLOR

	mov	hUpIcon, eax

	.if	(eax == NULL)			; failed

		invoke	GetLastError
		mov	G_APIErrCode, eax

		invoke	RaiseException, ERR_ImageLoad,
				0, nErrors, addr ErrorMsgs

	.endif


	invoke	LoadImage, G_hTui, ID_TM_DNICON, IMAGE_ICON, 0, 0,
			LR_DEFAULTCOLOR

	mov	hDnIcon, eax

	.if	(eax == NULL)			; failed

		invoke	GetLastError
		mov	G_APIErrCode, eax

		invoke	DestroyIcon, hUpIcon

		invoke	RaiseException, ERR_ImageLoad,
				0, nErrors, addr ErrorMsgs

	.endif


; Setup Time window.
;
; Creation notes:
;
; - WS_EX_TOOLWINDOW does not appear in the taskbar or in the dialog that
;   appears when the user presses ALT+TAB.
;
;   WS_EX_TOPMOST is necessary to overlay the taskbar.
;
;   See also the window focus note at the beginning of SBAR.ASM
;
; - As hbrBackground is NULL we must explicitly paint the background (see
;   WNDCLASS in api guide). Done when drawing back buffer in DrawTime.


	mov	wc.cbSize,		sizeof WNDCLASSEX
	mov	wc.style,		CS_DBLCLKS
	mov	wc.lpfnWndProc, 	offset TimeWndHdlr
	mov	wc.cbClsExtra,		NULL
	mov	wc.cbWndExtra,		NULL
	mov	eax,			G_hTui
	mov	wc.hInstance,		eax
	mov	wc.hIcon,		NULL
	mov	wc.hIconSm,		NULL
	invoke	LoadCursorA,		NULL, IDC_ARROW
	mov	wc.hCursor,		eax
	mov	wc.hbrBackground,	NULL
	mov	wc.lpszMenuName,	NULL
	mov	wc.lpszClassName,	offset szTimeClass

	invoke	RegisterClassExA, addr wc

	.if	(eax == NULL)		; Register Class failed

		invoke	GetLastError
		mov	G_APIErrCode, eax

		invoke	RaiseException, ERR_WndReg,
				0, nErrors, addr ErrorMsgs

	.endif


	invoke	CreateWindowExA, WS_EX_TOPMOST or WS_EX_TOOLWINDOW,
				addr szTimeClass,
				addr szTimeClass,
				WS_POPUP,
				0, 0, G_ScrnWid, G_ScrnHgt,
				NULL, NULL,
				G_hTui, NULL

	.if	(eax == NULL)		; Create failed

		invoke	GetLastError
		mov	G_APIErrCode, eax

		invoke	RaiseException, ERR_WndCreate,
				0, nErrors, addr ErrorMsgs

	.endif

	mov	hTimeWnd, eax


; Initialise Time display to the initial Search done during DB_Init.


	invoke	DB_GetSearch
	invoke	Time_SetItem, TSetFirst, 0, eax


; Exit.


	ret


Time_Init	endp
;----------------------------------------------------------------------------




;----------------------------------------------------------------------------
Time_DeInit	proc
;----------
;
; De-Initialise Time module objects.
;
;
; Entry,Exit	n/a
;----------------------------------------------------------------------------


	local	szKeyVal[16]		:byte


; Icons.


	.if	(hDnIcon)
		invoke	DestroyIcon, hDnIcon
	.endif

	.if	(hUpIcon)
		invoke	DestroyIcon, hUpIcon
	.endif


; Save Color set, but only if actually initialised.


	mov	eax, TM_OAF

	.if	(eax & TM_OAF_Colors)

		invoke	dw2hex, CTimeWnd, addr szKeyVal
		invoke	File_WrIniKey, addr szTMIS_COLR, addr szTMIK_TWBG,
				addr szKeyVal

		invoke	dw2hex, CTimeItem, addr szKeyVal
		invoke	File_WrIniKey, addr szTMIS_COLR, addr szTMIK_ITBG,
				addr szKeyVal

		invoke	dw2hex, CTimeStamp, addr szKeyVal
		invoke	File_WrIniKey, addr szTMIS_COLR, addr szTMIK_TSTX,
				addr szKeyVal

		invoke	dw2hex, CTimeBox, addr szKeyVal
		invoke	File_WrIniKey, addr szTMIS_COLR, addr szTMIK_TBBG,
				addr szKeyVal

	.endif


	ret


Time_DeInit	endp
;----------------------------------------------------------------------------




;----------------------------------------------------------------------------
Time_Color	proc	GetSet:dword, pColors:dword
;---------
;
; Get or Set Time window object colors, primarily for SBar module.
;
;
; Entry		GetSet		true to set, false to get
;
;		pColors		pointer to color table
;
;
; Exit		n/a
;----------------------------------------------------------------------------


	mov	edx, pColors

	mov	eax, GetSet

	.if	(eax == SetColors)

		mov	eax, [edx + TC_TWBG*4]
		mov	CTimeWnd, eax
		mov	eax, [edx + TC_ITBG*4]
		mov	CTimeItem, eax
		mov	eax, [edx + TC_TSTX*4]
		mov	CTimeStamp, eax
		mov	eax, [edx + TC_TBBG*4]
		mov	CTimeBox, eax

		; Change Time and Item backgrounds.

		invoke	DeleteObject, hBGBrush
		invoke	CreateSolidBrush, CTimeWnd
		mov	hBGBrush, eax

		invoke	DeleteObject, hItemBrush
		invoke	CreateSolidBrush, CTimeItem
		mov	hItemBrush, eax

	.else	; Get colors.

		mov	eax, CTimeWnd
		mov	[edx + TC_TWBG*4], eax
		mov	eax, CTimeItem
		mov	[edx + TC_ITBG*4], eax
		mov	eax, CTimeStamp
		mov	[edx + TC_TSTX*4], eax
		mov	eax, CTimeBox
		mov	[edx + TC_TBBG*4], eax

	.endif


	ret


Time_Color	endp
;----------------------------------------------------------------------------




;----------------------------------------------------------------------------
Time_Handle	proc
;----------
;
; Get Time window handle, primarily for SBar module.
;
;
; Entry		n/a
;
;
; Exit		eax	handle
;----------------------------------------------------------------------------


	mov	eax, hTimeWnd


	ret


Time_Handle	endp
;----------------------------------------------------------------------------




;----------------------------------------------------------------------------
Time_Stop	proc	Stop:dword
;--------
;
; Stop/Start the Time window updating on mouse movement or timer.
;
; Stop is required for the exception error MessageBox as it posts these
; messages to the Time window!.
;
; Also required when the Time window is not foremost for any reason (SideBar
; dialogs or Desktop window up) as otherwise the auto move timer is
; continually redrawing the window, in the background but in any case...
;
;
; Entry		Stop		TRUE to stop, FALSE to restart
;
;
; Exit		bTimeStop	TRUE if Stop true, FALSE if Stop false
;----------------------------------------------------------------------------


	mov	eax, Stop
	mov	bTimeStop, eax


	ret


Time_Stop	endp
;----------------------------------------------------------------------------




;----------------------------------------------------------------------------
Time_Destroy	proc
;-----------
;
; Destroy Time window.
;
; NB this cannot go in Time_DeInit as it uses the Tui message loop.
;
; Check handle is not zero jic we are coming here from the Fatal Execption
; handler and we are cleaning up after the user has dismissed the MessageBox.
;
;
; Entry,Exit	n/a
;
;
; DestroyWindow does not exit until all its messages are sent and processed.
;----------------------------------------------------------------------------


	.if	(hTimeWnd != 0)

		invoke	DestroyWindow, hTimeWnd

	.endif


	ret


Time_Destroy	endp
;----------------------------------------------------------------------------




;----------------------------------------------------------------------------
Time_GetItem	proc
;-----------
;
; Get the state of the current display.
;
;
; Entry		n/a
;
;
; Exit		eax	First Item
;		ecx	Current Item
;		edx	Search Items (LastItem+1)
;----------------------------------------------------------------------------


	mov	eax, SA_FItem
	mov	ecx, SA_CItem
	mov	edx, SA_NItems


	ret


Time_GetItem	endp
;----------------------------------------------------------------------------




;----------------------------------------------------------------------------
Time_SetItem	proc	Set:dword, Item:dword, nItems:dword
;-----------
;
; Set Time display to show from an item index into the Search array, making
; it the most recent on the display (closest). Reset the image cache.
;
;
; Entry/Exit	Set	how to set the display:
;
;		TSetFirst	set to first item
;
;			Entry:	Item		first Item
;				nItems		number of items in array
;
;			Exit:	SA_FItem	Item
;				SA_NItems	nItems
;				SrchItems	nItems
;				BaseItem	0
;				zAdj		+1.0
;				If Item !0 then Search array moved down
;
;		TSetLast	set to last item
;
;			Entry:	Item		n/a
;				nItems		0	don't reset SrchItems
;
;			Exit:	SA_NItems	nItems
;				SrchItems	unchanged
;				BaseItem	SrchItems - DispItems
;				zAdj		-(SrchItems-2)
;
;			Entry:	Item		n/a
;				nItems		<>0	reset SrchItems
;
;			Exit:	SA_NItems	nItems
;				SrchItems	nItems
;				BaseItem	nItems - DispItems
;				zAdj		-(nItems-2)
;
;		TSetItem	set to item
;
;			Entry:	Item		first item to display
;				nItems		n/a
;
;			Exit:	SA_CItem	Item
;				BaseItem	conditional
;				zAdj		-(Item-1)
;----------------------------------------------------------------------------


	mov	eax, Set


	.if	(eax == TSetFirst)


		; Adjust Search Array to begin at start item.

		mov	eax, Item
		mov	SA_FItem, eax

		.if	(eax)
			invoke	DB_SetSearch, eax, nItems
			mov	nItems, eax
		.endif

		; Set items in Search array.

		mov	eax, nItems
		mov	SrchItems, eax
		mov	SA_NItems, eax

		; Initialise the number of displayed items.
		;
		; If the Search array has less members than the default number
		; of displayed items then num. of displayed items is reduced.

		.if	(eax > DefDispItems)
			mov	eax, DefDispItems
		.endif

		mov	DispItems, eax

 		; Reset Base Item to start of Search array.

 		xor	eax, eax
		mov	BaseItem, eax

		; Initial z adjustment value is +1.0 - this makes the newest
		; item visible with a z value of 2 as DrawNdx decrements to 1
		; (see ShowTime).

		fld1
		fstp	zAdj


	.elseif	(eax == TSetLast)


		mov	eax, nItems
		mov	SA_NItems, eax

		.if	(eax)

			; Set items in Search array.

			mov	SrchItems, eax

			; Initialise the number of displayed items.
			; (as for TSetFirst above).

			.if	(eax > DefDispItems)
				mov	eax, DefDispItems
			.endif

			mov	DispItems, eax

		.endif

		; To show the last item as the only item on the display
		; we need to set the BaseItem to DispItems - 1 before the
		; end of the array and set the z adjustment to the end of
		; the array. This will push the displayed items down so
		; just the last is shown.
		;
		; We need to add 1 to the adjustment to ensure the item
		; is visible (z <- 2, see ShowTime).

		mov	eax, SrchItems
		sub	eax, DispItems
		mov	BaseItem, eax

		mov	eax, SrchItems
		dec	eax
		dec	eax			; [n - 1] == [-n + 1]
		mov	Item, eax
		fild	Item
		fchs				; (zAdj is negative)
		fstp	zAdj


	.elseif	(eax == TSetItem)


		; Display from Item.
		;
		; Similarly to TSetLast if the Item is within DispItems
		; of the end of the array, we need to freeze BaseItem
		; at SrchItems - DispItems and let the z adjustment
		; increase to bring the trailing end of the array down
		; the display.

		mov	eax, Item
		mov	SA_CItem, eax
		mov	edx, eax
		add	edx, DispItems

		.if	(edx >= SrchItems)

			; If we are in the last page of items or above we
			; need to set the BaseItem 1 page below.

			mov	edx, SrchItems
			sub	edx, DispItems

		.else

			; Else we are in range 0 to (SrchItems-DispItems)
			; so the BaseItem becomes the entered item.

			mov	edx, eax

		.endif

		mov	BaseItem, edx

		; Set the z adjustment.

		dec	Item
		fild	Item
		fchs				; (zAdj is negative)
		fstp	zAdj


	.endif


; Reset the Image and Text caches.
;
; [Qry plugin caches?]
;
; Set Item field to -1 to denote not loaded.


	xor	eax, eax
	dec	eax
	mov	ecx, nImgObj-1

	.repeat

		push	ecx
		shl	ecx, ImgObjRecMult
		mov	ImgObj[ecx].Item, eax
		pop	ecx
		dec	ecx

	.until	(SIGN?)

	mov	ecx, nTxtObj-1

	.repeat

		push	ecx
		shl	ecx, TxtObjRecMult
		mov	TxtObj[ecx].Item, eax
		pop	ecx
		dec	ecx

	.until	(SIGN?)


	ret


Time_SetItem	endp
;----------------------------------------------------------------------------




;----------------------------------------------------------------------------
Time_TogThumb	proc
;------------
;
; Toggle Thumbnailing on/off and update Time window.
;
;
; Entry,Exit	n/a
;----------------------------------------------------------------------------


	xor	DoThumb, 1
	invoke	ShowTime


	ret


Time_TogThumb	endp
;----------------------------------------------------------------------------




;============================================================================
